package platform

import (
	"applet/app/db"
	"applet/app/md"
	"applet/app/utils"
	"github.com/gin-gonic/gin"
	"time"
)

func OrderList(c *gin.Context, args map[string]string) ([]map[string]string, int64) {

	masterId, _ := c.Get("master_id")
	playletSaleOrderDb := db.PlayletSaleOrderDb{}
	playletSaleOrderDb.Set()
	args["mid"] = utils.AnyToString(masterId)
	args["sort"] = "create_time desc,id desc"
	list, total := playletSaleOrderDb.GetPlayletVideoOrderListAndTotal(args)
	var data = make([]map[string]string, 0)
	if list != nil {
		for _, v := range *list {
			source := md.OrdTypeMap[v.OrdType] + md.VideoTypeMap[v.VideoType]
			settleStatus := "未结算"
			settleTime := "-"
			if v.SettleTime > 0 {
				settleStatus = "已结算"
				settleTime = time.Unix(int64(v.SettleTime), 0).Format("2006-01-02 15:04:05")
			}
			var tmp = map[string]string{
				"oid":                  v.Oid,
				"mid":                  v.Uid,
				"custom_oid":           v.CustomOid,
				"title":                v.Title,
				"source":               source,
				"amount":               v.Amount,
				"platform_fee":         v.PlatformFee,
				"commission":           v.Commission,
				"real_commission":      v.Commission,
				"status":               v.Status,
				"settle_status":        settleStatus,
				"platform_settle_time": "-",
				"create_time":          v.CreateTime.Format("2006-01-02 15:04:05"),
				"settle_time":          settleTime,
				"commission_bili":      "-",
			}
			if utils.StrToFloat64(v.Commission) > 0 {
				tmp["commission_bili"] = utils.Float64ToStr(utils.StrToFloat64(v.Commission)/utils.StrToFloat64(v.Amount)*100) + "%"
			}
			data = append(data, tmp)
		}
	}
	return data, total
}

func OrderOutput(c *gin.Context, args map[string]string) (string, map[string]string) {

	masterId, _ := c.Get("master_id")
	playletSaleOrderDb := db.PlayletSaleOrderDb{}
	playletSaleOrderDb.Set()
	args["mid"] = utils.AnyToString(masterId)
	args["size"] = "3000"
	if args["ids"] != "" {
		args["size"] = "0"
	}
	name := "订单_" + args["p"]
	//写入数据
	data := map[string]string{
		"A1": "订单号",
		"B1": "会员ID",
		"C1": "标题",
		"D1": "来源",
		"E1": "结算时间",
		"F1": "订单状态",
		"G1": "金额",
		"H1": "佣金",
	}
	list := playletSaleOrderDb.GetPlayletVideoOrderListWithCreateTime(args)
	if list != nil {
		for k, v := range *list {
			source := md.OrdTypeMap[v.OrdType] + md.VideoTypeMap[v.PlatformType]
			settleTime := "-"
			if v.SettleTime > 0 {
				settleTime = time.Unix(int64(v.SettleTime), 0).Format("2006-01-02 15:04:05")
			}
			i := utils.IntToStr(k + 2)
			data["A"+i] = v.Oid
			data["B"+i] = utils.IntToStr(v.SubUid)
			data["C"+i] = v.Title
			data["D"+i] = source
			data["E"+i] = settleTime
			data["F"+i] = v.Status
			data["G"+i] = v.Amount
			data["H"+i] = v.Commission
		}
	}
	return name, data
}
