package svc

import (
	"applet/app/db"
	"applet/app/db/model"
	offical "applet/app/db/official"
	"applet/app/task/md"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_third_party_api.git/tik_tok"
	zhios_third_party_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_third_party_api.git/utils"
	"encoding/json"
	"fmt"
	"github.com/tidwall/gjson"
	"strings"
	"time"
)

func TikTokLocalLifeOrder() {
	all := offical.MasterListCfgGetKeyAll("tik_tok_open_appkey")
	if all == nil {
		return
	}
	for _, v := range *all {
		if v.V == "" {
			continue
		}
		data := offical.MasterListCfgGetOneData(v.Uid, "tik_tok_open")
		if data != "1" && utils.StrToInt(v.Uid) > 0 {
			continue
		}
		p := ""
		for true {
			_, curor, hasMore := GetTikTokLocalLifeOrder(p, v.Uid)
			if hasMore == false {
				break
			}
			p = curor
		}
	}

}
func TikTokToken(uid string) string {
	tikTokOpenAppkey := offical.MasterListCfgGetOneData(uid, "tik_tok_open_appkey")
	tikTokOpenAppSecret := offical.MasterListCfgGetOneData(uid, "tik_tok_open_app_secret")
	args := map[string]string{
		"appkey":    tikTokOpenAppkey,
		"appSecret": tikTokOpenAppSecret,
	}
	key := "tikTok_client_" + args["appkey"]
	tokeStr, err := cache.GetString(key)
	if tokeStr == "" || err != nil {
		token := tik_tok.GetClientToken(args)
		if token["tik_tok_acc_token"] != "" {
			tokeStr = token["tik_tok_acc_token"]
			cache.SetEx(key, token["tik_tok_acc_token"], utils.StrToInt(token["expires_in"]))
		}
	}
	return tokeStr
}

func GetTikTokLocalLifeOrder(p string, uid string) (int, string, bool) {
	tikTokOpenUid := offical.MasterListCfgGetOneData(uid, "tik_tok_open_uid")

	endTime := time.Now().Unix()
	startTime := endTime - 86400
	param := map[string]interface{}{
		"uid":               tikTokOpenUid,
		"cursor":            p,
		"count":             "50",
		"update_time_begin": startTime,
		"update_time_end":   endTime,
	}
	fmt.Println(param)
	api, _ := tik_tok.LocalApi("query_order/", TikTokToken(uid), zhios_third_party_utils.SerializeStr(param))
	count := 0
	data := gjson.Get(api, "data").String()
	if data != "" {
		var list = md.TikTokLocalLifeOrder{}
		json.Unmarshal([]byte(data), &list)
		count = len(list.ItemOrders)
		for _, v := range list.ItemOrders {
			AddTikTokLocalLifeOrder(v)
		}
		return count, list.Cursor, list.HasMore
	}
	return count, "", false
}
func AddTikTokLocalLifeOrder(order md.TikTokLocalLifeOrderDetail) {

	statusArr := map[int]string{1: "订单付款", 2: "订单退款", 3: "订单成功", 4: "部分核销", 5: "部分退款"}
	userId := order.CommissionInfo.DistributionInfo.ExternalInfo
	isShare := 0
	if strings.Contains(utils.AnyToString(userId), "s") {
		isShare = 1
	}
	split := strings.Split(utils.AnyToString(userId), "_")
	if len(split) < 3 {
		return
	}
	mid := split[2]
	puid := ""
	if len(split) == 4 {
		puid = split[2]
	}
	uid := split[1]
	title := ""
	img := ""
	if len(order.ProductList) > 0 {
		title = order.ProductList[0].Name
		img = order.ProductList[0].Img
	}
	var ord = &model.GuideOrder{
		Oid:          utils.StrToInt64(utils.OrderUUID(utils.StrToInt(uid))),
		Uid:          utils.StrToInt(mid),
		ExtendUid:    utils.StrToInt(puid),
		StationUid:   utils.StrToInt(uid),
		PvdOid:       order.ItemOrderId,
		PvdParentOid: order.RootOrderInfo.RootOrderId,
		Status:       statusArr[order.Status],
		CreateTime:   order.PayTime,
		UpdateTime:   int(time.Now().Unix()),
		Commission:   utils.Float64ToStr(float64(order.CommissionInfo.EstimateCommission) / 100),
		Title:        title,
		Payment:      utils.Float64ToStr(float64(order.PayAmount) / 100),
		Pvd:          "tikTok_life",
		Img:          img,
		IsShare:      isShare,
	}

	if order.CommissionInfo.AllSettleRemitFinish || order.Status != 2 {
		var amount int64 = 0
		isEnd := 1
		var endTime int64 = 0
		for _, v := range order.CommissionInfo.SettleInfoMap {
			tmp, ok := v.(map[string]interface{})
			if ok == false || utils.AnyToInt64(tmp["status"]) < 3 {
				isEnd = 0
			}
			if ok && utils.AnyToInt64(tmp["status"]) == 3 {
				endTime = utils.AnyToInt64(tmp["settle_time"])
				amount += utils.AnyToInt64(tmp["settle_amount"])
			}
		}
		if len(order.CommissionInfo.SettleInfoMap) == 0 {
			isEnd = 0
		}
		if isEnd == 1 {
			ord.Status = "订单结算"
			if order.Status == 5 {
				ord.Status = "部分结算"
			}
			ord.PlatformSettleTime = int(endTime)
			if ord.PlatformSettleTime == 0 {
				ord.PlatformSettleTime = ord.UpdateTime
			}
			ord.RealCommission = utils.Float64ToStr(float64(amount) / 100)
		}
	}
	one := db.GetGuideOrderByOne(ord.PvdOid, utils.IntToStr(ord.Uid), ord.Pvd)
	if one == nil {
		insertOne, err := db.ZhimengDb.InsertOne(ord)
		fmt.Println(insertOne)
		fmt.Println(err)
	} else {
		ord.SettleTime = one.SettleTime
		if one.PlatformSettleTime > 0 {
			ord.PlatformSettleTime = one.PlatformSettleTime
		}
		db.ZhimengDb.Where("id=?", one.Id).AllCols().Update(ord)
	}
	return
}
